#! usr/bin/python
# -*- coding: ISO-8859-1 -*-

import numpy as np
import cv2 as cv
import itertools

from KP_tools import *
    

def eyesKPDetection(x,y,w,h,face_gray,img,sift,show_zone=True):
    """
    Fonction de detection des points-cles des yeux
    """
    
    eye_cascade = cv.CascadeClassifier('../xml_files/haarcascade_eye.xml')
    eyes = eye_cascade.detectMultiScale(face_gray, 1.3, 5)
    
    if len(eyes) == 0:
        print('ATTENTION AUCUN OEIL DETECTE')
        return []
    
    eyes_KP = []
    if len(eyes) == 1:
        valid_eyes = [eyes[0,]]
    else:
        valid_eyes = [eyes[0,],eyes[1,]]
    for (ex,ey,ew,eh) in valid_eyes:

        sub_eyes = face_gray[ey:ey+eh, ex:ex+ew]
        # detection des points cles presents dans la sous-zone des yeux
        detection_y = sift.detect(sub_eyes,None)
        detection_y = correctionCoords(detection_y,(x+ex,y+ey))
        eyes_KP.append(detection_y)
        
        if show_zone:
            # tracage d'un rectangle delimitant la zone des yeux detectes
            cv.rectangle(img,(ex+x,ey+y),(ex+x+ew,ey+y+eh),(255,0,0),2)
    
    return list(itertools.chain.from_iterable(eyes_KP)) # fusion des sous-listes
    
    

def noseKPDetection (x,y,w,h,face_gray,img,sift,show_zone=True):
    """
    Fonction de detection des points-cles du nez
    """
    
    nose_cascade = cv.CascadeClassifier('../xml_files/haarcascade_mcs_nose.xml')
    nose = nose_cascade.detectMultiScale(face_gray, 1.3, 5) # detection du nez sur le visage detecte
    
    if len(nose) == 0:
        print('ATTENTION AUCUN NEZ DETECTE')
        return []
    
    valid_nose = [nose[0,]]
    for (nx,ny,nw,nh) in valid_nose:
        
        sub_nose = face_gray[ny:ny+nh, nx:nx+nw]
        # detection des points cles presents dans la sous-zone du nez
        nose_KP = sift.detect(sub_nose,None)
        
        if show_zone:
            # tracage d'un rectangle delimitant la zone du nez detecte
            cv.rectangle(img,(nx+x,ny+y),(nx+x+nw,ny+y+nh),(0,0,255),2) # trace un rectangle autour du nez
    
        return correctionCoords(nose_KP, (x+nx,y+ny))


def mouthKPDetection(x,y,w,h,face_gray,img,sift,show_zone=True):
    """
    Fonction de detection des points-cles de la bouche
    """
    
    mouth_cascade = cv.CascadeClassifier('../xml_files/Mouth.xml')
    mouth = mouth_cascade.detectMultiScale(face_gray, 1.3, 5) # detection de la bouche sur le visage detecte
    
    if len(mouth) == 0:
        print('ATTENTION AUCUNE BOUCHE DETECTEE')
        return []
    
    for (mx,my,mw,mh) in mouth:
        
        #if my+30 > ny+nh:
        if my > h *0.6:
            sub_mouth = face_gray[my:my+mh, mx:mx+mw]
            # detection des points cles presents dans la sous-zone de la bouche
            mouth_KP = sift.detect(sub_mouth,None)
            
            if show_zone:
                # tracage d'un rectangle delimitant la zone de la boucehe detectee
                cv.rectangle(img,(mx+x,my+y),(mx+x+mw,my+y+mh),(0,255,0),2) # trace un rectangle autour de la bouche
                
            return correctionCoords(mouth_KP, (x+mx,y+my))
    
    print('AUCUNE BOUCHE SATISFAISANTE DETECTEE [position y]')
    return []
    


def keyPointsDetection (x,y,w,h,img,show_zone=True):

    # passage de l'image en valeurs de gris
    grayImg = cv.cvtColor(img,cv.COLOR_BGR2GRAY)


    # initialisation de l'algorithme de detection de points cles SIFT #
    sift = cv.xfeatures2d.SIFT_create(nfeatures=25, contrastThreshold=0.004)

    # tracage d'un rectangle delimitant la zone du visage detecte
    cv.rectangle(img,(x,y),(x+w,y+h),(255,0,0),2)
    # determination d'une sous-zone visage, dans laquelle vont se faire les detections
    # des elements du visage actuel (yeux, nez, bouche)
    face_gray = grayImg[y:y+h, x:x+w]

    eyes_KP = eyesKPDetection(x,y,w,h,face_gray,img,sift,show_zone)
    nose_KP = noseKPDetection(x,y,w,h,face_gray,img,sift,show_zone)
    mouth_KP = mouthKPDetection(x,y,w,h,face_gray,img,sift,show_zone)
    
    print('KP: %d YEUX / %d NEZ / %d BOUCHE' %(len(eyes_KP),len(nose_KP),len(mouth_KP)))
    
    key_points = [mouth_KP, nose_KP, eyes_KP]
    return list(itertools.chain.from_iterable(key_points))
