#! usr/bin/python
# -*- coding: ISO-8859-1 -*-

import numpy as np
import cv2 as cv


def convertListToKP (list):
    sortie = []
    for pt in list:
        sortie.append(cv.KeyPoint(pt[0], pt[1],2))
    return sortie


def createCirclePoints (X, Y, radius, nb_points):
    x = np.linspace(0,2*np.pi,nb_points)
    return np.transpose(radius * np.cos(x)+X), np.transpose(radius * np.sin(x) + Y)


def circleCoordonates(x,y,w,h,nbc):
    center_x = x+(w/2)
    center_y = y+(h/2)
    size_c = w/2
    x,y = createCirclePoints(center_x,center_y,size_c,nbc)
    pts = []
    pts.append(x)
    pts.append(y)
    pts = np.transpose(pts)
    return convertListToKP(pts)



def distanceBetweenPoints (p1, p2):
    """
    Calcule et retourne la distance euclidienne entre deux points [x1,y1] et [x2,y2]
    """
    return np.sqrt(pow(p2[0] - p1[0],2) + pow(p2[1] - p1[1],2))


def computeDistances (kp):
    """
    Creation d'une liste 1D, dont chaque element correspond a la distance euclidienne
    entre un point donne et le point (0,0), et retourne la liste
    """
    nb_points = len(kp)
    distances = np.zeros(nb_points)

    for i in range(nb_points):
        distances[i] = distanceBetweenPoints((0,0), kp[i].pt)
    return distances


def orderDistances (index, list_coords):
    """
    Trie une liste de coordonnees selon une liste d'indices de position passee en
    parametre (index), et retourne cette liste
    """
    nb_points = len(list_coords)
    sorted_pts = [0] * nb_points

    for i in range(nb_points):
        sorted_pts[index[i]] = list_coords[i]
    return sorted_pts


def sortDistances (list_coords, kp):
    """
    Trie un tableau de coordonnees selon leur distance a l'origine. Retourne une liste
    ordonnee de coordonnees
    """
    dist = computeDistances(kp)
    sort_index = np.argsort(dist)
    return orderDistances(sort_index,list_coords)


def getPointsInZone (kp, top_left, dim):
    """
    Fonction parcourant une liste de points cles, et renvoie une liste de points cles
    appartenant a la zone delimitee par la zone definie par le point top_left, et
    de taille size
    """
    # NOTE : possibilite de vider la liste pour eviter les redondances ? #
    points_list = []
    for point in kp:
        if (top_left[0] <= point.pt[0] <= top_left[0]+dim[0]) and (top_left[1] <= point.pt[1] <= top_left[1]+dim[1]):
            points_list.append(point)
    return points_list


def printKeyPointsList (kp):
    """
    Affichage des coordonnees de la liste des KeyPoint passee en parametre
    """
    for point in kp:
        print(point.pt)
    print('%d points\n'%len(kp))


def correctionCoords (kp, corner_xy):
    """
    Correction des coordonnees des KeyPoint, permettant de faire passer leur coordonnees relatives
    a la zone dans laquelle ils ont ete extraits, a une position globale dans l'image. Retourne
    cette correction sous forme d'une nouvelle liste
    """
    new_coords = []
    for point in kp:
        new_coords.append(cv.KeyPoint(point.pt[0]+corner_xy[0], point.pt[1]+corner_xy[1],2))
    return new_coords


def approximateCoords (kp):
    """
    Permet d'arrondir les coordonnees d'une liste de KeyPoint a la coordonnee la plus proche.
    Retourne cette arrondi sous forme d'une nouvelle liste """
    new_coords = []
    for point in kp:
        new_coords.append(cv.KeyPoint(int(point.pt[0]+0.5), int(point.pt[1]+0.5),2))
    return new_coords


def writeKeyPoints (filename, kp):
    """
    Ecriture des coordonnees de points cles (KeyPoint) dans un fichier designe par filename
    (SANS EXTENSION !), la fonction retourne la liste de coordonnees creee
    """
    kp_l = len(kp)
    coord_x = np.zeros(kp_l, dtype=np.int)
    coord_y = np.zeros(kp_l, dtype=np.int)
    for i in range(kp_l):
        coord_x[i] = kp[i].pt[0]
        coord_y[i] = kp[i].pt[1]

    cd = np.column_stack((coord_x,coord_y))
    np.savetxt('../txt_files/' + filename + '_detect.txt', cd, fmt='%i')
    return cd
