#! usr/bin/python
# -*- coding: ISO-8859-1 -*-

# Projet de réalisation technique : Les nano-ordinateurs pour une application d'intelligence artificielle #
# BARRE Maxime: maxime.barre@etu.univ-amu.fr
# BUSQUET Michael: michael.busquet@etu.univ-amu.fr
# EIXARCH Aymeric: aymeric.eixarch@etu.univ-amu.fr
# Master 1 Traitement du signal et des images, Université d'Aix-Marseille #

"""
NOTE IMPORTANTE : pour regler les problemes de repartition des zones, on peut utiliser
la proportionnalite des elements du visage (yeux au tier, nez au milieu, bouche
aux deux tiers ...)
"""


import sys
import matplotlib.pyplot as plt
import numpy as np
import cv2 as cv
import itertools
from scipy.spatial import Delaunay

from KP_tools import *
from KP_detection import *
from triangulation import *
from wrapping import *
from utils import *


def computeProgram (img_name):
    # chargement de l'image cible
    img = cv.imread('../img_files/' + img_name + '.jpg')

    face_cascade = cv.CascadeClassifier('../xml_files/haarcascade_frontalface_default.xml')
    faces = face_cascade.detectMultiScale(img, 1.3, 5)
    
    if len(faces) == 0:
        print('ATTENTION AUCUN VISAGE DETECTE !')
        sys.exit('no face')
    
    kp_lists = [] # la liste des points cles (KeyPoint)
    kp_coords = [] # la liste des tuples de coordonnees
    
    for count,(x,y,w,h) in enumerate(faces):

        # on detecte les points cles sur chaque visage
        KP = keyPointsDetection(x,y,w,h,img,show_zone=True)
        detected_kp = approximateCoords(KP)
        # on cree le cercle englobant le visage
        #circle_kp = circleCoordonates(x,y,w,h,30)
        #detected_kp = np.concatenate([detected_kp,circle_kp])
        print('%d KEY POINTS'%len(detected_kp))
        # on ecrit les coordonnees des KeyPoint dans des .txt
        coords = writeKeyPoints(img_name+str(count), detected_kp)
        kp_coords.append(coords)

        
        # on cree une triangulation de Delaunay a partir des KeyPoint
        delaunay = Delaunay(coords)
        print('%d TRIANGLES'%len(delaunay.simplices))
        # dessin de la triangulation sur l'image
        drawTriangulation(img,coords,delaunay)
        
        # dessin des points cles sur l'image
        
        imgKP = cv.drawKeypoints(img,detected_kp,img)
        kp_lists.append(detected_kp)

    # pour l'instant, kp_lists et kp_coords sont inutiles mais on leur trouvera surement
    # une utilite apres !

    '''
    # tri des distances en fonction de la distance a l'origine
    sorted_dist = sortDistances(kp_coords,key_points)
    '''
    
    cv.imwrite('../img_files/'+img_name+'_detect.jpg', imgKP)

    return delaunay
    


if __name__ == '__main__':

    img1_name = 'test'
    img2_name = 'manu'
    
    d1 = computeProgram(img1_name)
    d2 = computeProgram(img2_name)
    
    transf = 
